from nose.tools import *
from hawkweed.functional.list_prims import *

def test_first():
    assert_equal(first([1]), 1)

def test_rest():
    assert_equal(rest([1, 2, 3]), [2, 3])

def test_second():
    assert_equal(second([1, 2]), 2)

def test_last():
    assert_equal(last([1, 2, 3]), 3)

def test_get():
    assert_equal(get(2, [1, 2, 3]), 3)

def test_remove_from():
    x = [1, 2, 3]
    assert_equal(remove_from(2, x), [1, 2])
    assert_equal(x, [1, 2])

def test_remove_from_keep():
    x = [1, 2, 3]
    assert_equal(remove_from_keep(2, x), [1, 2])
    assert_equal(x, [1, 2, 3])

def test_aperture():
    x = [1, 2, 3]
    assert_equal(list(aperture(2, x)), [[1, 2], [2, 3]])

def test_aperture_empty():
    assert_equal(list(aperture(2, [])), [])

def test_aperture_bigger_n():
    assert_equal(list(aperture(2, [1])), [])



"""The Representation base class."""
class Repr(object):
    """The Representation base class."""
    def __repr__(self):
        return "{}({})".format(self.__class__.__name__, super(Repr, self).__repr__())

    def __str__(self):
        return self.__repr__()



"""A collection of useful functions"""
from hawkweed.functional.primitives import *
from hawkweed.functional.logical import *
from hawkweed.functional.mathematical import *
from hawkweed.functional.list_prims import *



from nose.tools import *
from hawkweed.functional.mathematical import *

def test_inc():
    for i in range(100):
        assert_equal(inc(i), i + 1)

def test_dec():
    for i in range(100):
        assert_equal(dec(i), i - 1)

def test_add():
    assert_equal(add(1)(2), 3)

def test_sub():
    assert_equal(sub(1)(2), -1)

def test_mul():
    assert_equal(mul(1)(2), 2)

def test_div():
    assert_equal(div(10)(10), 1)

def test_mod():
    assert_equal(mod(3)(2), 1)

def test_clamp():
    assert_equal(clamp(1, 10, 3), 3)
    assert_equal(clamp(1, 10, -3), 1)
    assert_equal(clamp(1, 10, 30), 10)



"""The identity monad"""
from hawkweed.monads.monad import Monad

class Identity(Monad):
    """The identity monad"""
    def bind(self, fun):
        """
        The monadic bind function of Identity.
        It will just apply the function to the
        value and be done with it.

        Complexity: O(k) where k is the complexity of the function
        params:
            fun: the function
        returns: the transformed value
        """
        return fun(self.value)



"""An augmented version of the list class"""
from hawkweed.classes.iterable import Iterable
from hawkweed.classes.repr import Repr
from hawkweed.classes.collection import Collection

class List(Repr, list, Iterable, Collection):
    """An augmented version of the list class"""
    def reset(self, other):
        """
        Resets a list to another lists content

        Complexity: O(n+m)
        params:
            other: the list this list should be resetted to
        returns: self
        """
        self.clear()
        self.extend(other)
        return self

    def remove_empty(self, fun=None):
        """
        Removes empty elements from the list.

        Complexity: O(3*n)
        params:
            fun: a function that takes an element and returns whether it
                 should be kept (defaults to bool())
        returns: self
        """
        if not fun:
            fun = bool

        tmp = []
        for elem in self:
            if fun(elem):
                tmp.append(elem)

        self.reset(tmp)
        del tmp
        return self

    def rindex(self, elem):
        """
        Gets the index of an element starting from the end. Does not copy the list.

        Complexity: O(n)
        params:
            elem: the element that should be found
        returns: self
        """
        for i, find in enumerate(reversed(self)):
            if elem == find:
                return i
        return -1

    def take(self, num):
        """
        A generator that returns a subarray (from 0 to num).

        Complexity: O(n+k) where k is the param
        params:
            num: the upper limit of the generator slice
        """
        for i, elem in enumerate(self):
            if num <= i:
                break
            yield elem

    def take_while(self, fun):
        """
        A generator that returns elements while a given function fun returns True.

        Complexity: O(n)
        params:
            fun: the predicate function
        """
        for elem in self:
            if fun(elem):
                yield elem
            else:
                break

    def drop(self, num):
        """
        A generator that returns a subarray (from num to len(List)).

        Complexity: O(n+k) where k is the param
        params:
            num: the lower limit of the generator slice
        """
        for elem in self[num:]:
            yield elem

    def drop_while(self, fun):
        """
        A generator that skips elements while a given function fun returns True.

        Complexity: O(n+k)
        params:
            fun: the predicate function
        """
        i = 0
        for i, elem in enumerate(self):
            if not fun(elem):
                break

        for elem in self[i:]:
            yield elem

    def clear(self):
        """
        A reimplemented version of clear because it only exists in Python3.

        Complexity: O(n)
        returns: self
        """
        del self[:]
        return self

    def append(self, *args, **kwargs):
        """
        Augmented update function that returns self.

        returns: self
        """
        super(List, self).append(*args, **kwargs)
        return self

    def extend(self, *args, **kwargs):
        """
        Augmented extend function that returns self.

        returns: self
        """
        super(List, self).extend(*args, **kwargs)
        return self

    def get(self, index, dflt=None):
        """
        A getter function that behaves like dict.get.

        params:
            index: the index to get
            dflt: the default return value (defaults to None)
        returns: self
        """
        if len(self) > index:
            return self[index]
        return dflt



"""An Iterable baseclass"""

class Iterable():
    """An Iterable baseclass"""
    def __iter__(self):
        """This is the only functions that needs to be implemented to be an Iterable."""
        raise NotImplementedError("Iterable subclasses need to implement this function")

    def concatenate(self, *arguments):
        """
        Concatenates multiple Iterables.

        Complexity: O(1)
        params:
            *arguments: multiple Iterables
        returns: the new Iterable
        """
        def internal():
            """the internal concatenater"""
            for i in [self] + list(arguments):
                for elem in i:
                    yield elem
        return internal()

    def nth(self, num):
        """
        Returns an Iterable that will only yield every nth value.

        Complexity: O(1)
        params:
            num: the step size
        returns: the new Iterable
        """
        if num < 1:
            raise ValueError("step size must be bigger than 0, was {}".format(num))
        def internal():
            """the internal iterator"""
            for i, elem in enumerate(self):
                if i % num == 0:
                    yield elem
        return internal()



"""The IO and CachedIO monads"""
from hawkweed.functional.primitives import apply
from hawkweed.monads.monad import Monad

class IO(Monad):
    """The IO monad"""
    def __init__(self, value, *args):
        self.value = value
        self.args = args

    def ret(self):
        return apply(self.value, self.args)

    def bind(self, fun):
        return fun(apply(self.value, self.args))

class CachedIO(IO):
    """The CachedIO monad"""
    def __init__(self, value, *args):
        self.value = value
        self.args = args
        self.res = None
        self.evald = False

    def _get_cache(self):
        if self.evald:
            return self.res
        self.res = apply(self.value, self.args)
        self.evald = True
        return self.res

    def ret(self):
        return self._get_cache()

    def bind(self, fun):
        return fun(self._get_cache())



"""The Either monad"""
from hawkweed.monads.monad import Monad

class Either(Monad):
    """The Either abstract base class"""
    def __init__(self, value):
        raise NotImplementedError("please use the type instances Left or Right")

class Left(Either):
    """The Left instance of the Either monad"""
    def __init__(self, value):
        self.value = value

    def bind(self, fun):
        """
        The monadic bind function of Left.
        It will just return itself.

        Complexity: O(1)
        params:
            fun: the function
        returns: self
        """
        return self

class Right(Either):
    """The Right instance of the Either monad"""
    def __init__(self, value):
        self.value = value

    def bind(self, fun):
        """
        The monadic bind function of Right.
        It will just apply the function to the
        value and be done with it.

        Complexity: O(k) where k is the complexity of the function
        params:
            fun: the function
        returns: the transformed value
        """
        return fun(self.value)

def either(left, right, monad):
    """
    Takes a function left, a function right and a value
    and binds according to the value (into left if it is a Left,
    into right if it is a Right). Otherwise throws a ValueError.

    Complexity: O(1) or complexity of the given function
    params:
        left: the function that should be executed on Left
        right: the function that should be executed on Right
    throws: ValueError
    returns:
        whatever the functions return
    """
    if is_left(monad):
        return monad.bind(left)
    if is_right(monad):
        return monad.bind(right)
    raise ValueError("monad in either must either be left or right")

def lefts(monads):
    """
    Takes a list and returns only the instances of Left.

    Complexity: O(1)
    params:
        monads: the list
    returns:
        an iterable of the Left values
    """
    return (x for x in monads if is_left(x))

def rights(monads):
    """
    Takes a list and returns only the instances of Right.

    Complexity: O(1)
    params:
        monads: the list
    returns:
        a generator of the Right values
    """
    return (x for x in monads if is_right(x))

def is_either(monad):
    """
    Checks whether a value is an instance of Either.

    Complexity: O(1)
    params:
        val: the value to check
    returns: the truth value
    """
    return is_left(monad) or is_right(monad)

def is_left(monad):
    """
    Checks whether a value is an instance of Left.

    Complexity: O(1)
    params:
        val: the value to check
    returns: the truth value
    """
    return isinstance(monad, Left)

def is_right(monad):
    """
    Checks whether a value is an instance of Right.

    Complexity: O(1)
    params:
        val: the value to check
    returns: the truth value
    """
    return isinstance(monad, Right)

def partition_eithers(monads):
    """
    Takes a list and returns a two-element Atuple where the first
    element is a list of all the instances of Left and the second
    element is a list of all the instances of Right.

    Complexity: O(1)
    params:
        monads: the list of monads
    returns: the tuple
    """
    return lefts(monads), rights(monads)



"""The Maybe monad"""
from hawkweed.functional.list_prims import first
from hawkweed.monads.monad import Monad

class Maybe(Monad):
    """The Maybe abstract base class"""
    def __init__(self, value):
        raise NotImplementedError("please use the type instances Just or Nothing")

class Just(Maybe):
    """The Just monad"""
    def __init__(self, value):
        self.value = value

    def bind(self, fun):
        """
        The monadic bind function of Just.
        It will just apply the function to the
        value and be done with it.

        Complexity: O(k) where k is the complexity of the function
        params:
            fun: the function
        returns: the transformed value
        """
        return fun(self.value)

class Nothing(Maybe):
    """The Nothing monad"""
    def __init__(self):
        self.value = None

    def bind(self, fun):
        """
        The monadic bind function of Nothing.
        It will just return itself.

        Complexity: O(1)
        params:
            fun: the function
        returns: self
        """
        return self

def is_maybe(val):
    """
    Checks whether a value is an instance of Maybe.

    Complexity: O(1)
    params:
        val: the value to check
    returns: the truth value
    """
    return is_just(val) or is_nothing(val)

def is_just(val):
    """
    Checks whether a value is an instance of Just.

    Complexity: O(1)
    params:
        val: the value to check
    returns: the truth value
    """
    return isinstance(val, Just)

def is_nothing(val):
    """
    Checks whether a value is an instance of Nothing.

    Complexity: O(1)
    params:
        val: the value to check
    returns: the truth value
    """
    return isinstance(val, Nothing)

def from_just(val):
    """
    returns the value of a just monad; otherwise
    throws a ValueError.

    Complexity: O(1)
    params:
        val: the monad to get the value from
    throws: ValueError
    returns: the monad value
    """
    if is_just(val):
        return val.value
    raise ValueError("val is not an instance of Just")

def from_maybe(val, dflt=None):
    """
    returns the value of a maybe monad; otherwise
    throws a ValueError.
    (catamorphism function)

    Complexity: O(1)
    params:
        val: the monad to get the value from
        dflt: the default value (defaults to None)
    throws: ValueError
    returns: the monad value
    """
    if is_just(val):
        return val.value
    elif is_nothing(val):
        return dflt
    else:
        raise ValueError("val is not an instance of Maybe")

def list_to_maybe(val):
    """
    if the list is empty, return Nothing. Otherwise
    return Just with the first element of the list.

    Complexity: O(1)
    params:
        val: the list fo build a Maybe from
    throws: ValueError
    returns: the monad
    """
    if not isinstance(val, list):
        raise ValueError("val is not an instance of list")
    if val == []:
        return Nothing()
    return Just(first(val))

def maybe_to_list(val):
    """
    if the value is nothing, return an empty list.
    Otherwise return a list with the monad value
    as single element.

    Complexity: O(1)
    params:
        val: the monad to build the list from
    throws: ValueError
    returns: list
    """
    if is_nothing(val):
        return []
    elif is_just(val):
        return [val.value]
    else:
        raise ValueError("val is not an instance of Maybe")

def value_to_maybe(val):
    """
    if the value is falsy, return Nothing. Otherwise
    return Just with the value as value.

    Complexity: O(1)
    params:
        val: the value to build a Maybe from
    returns: the monad
    """
    if not val:
        return Nothing()
    return Just(val)

def cat_maybe(vals):
    """
    Takes a list of Maybes and returns a generator that produces
    all the Just values.

    Complexity: O(n)
    params:
        vals: the list of maybes
    returns:
        a generator producing all the Just values
    """
    return (val for val in vals if is_just(val))

def map_maybe(fun, vals):
    """
    Takes a function and a list of elements and maps the function
    over them. It returns a generator that produces all the values
    that are Just after applying the function.

    Complexity: O(n)
    params:
        fun: the function to apply
        vals: the list of values
    returns:
        a generator producing all the just values
    """
    for element in vals:
        processed = fun(element)
        if is_just(processed):
            yield processed



from nose.tools import *
from hawkweed.monads.continuation import Continuation, callcc

def test_call():
    assert_equal(Continuation(lambda x: x + 10)(10), 20)

def test_bind():
    assert_equal(Continuation(lambda x: x)(lambda x: x + 10)(10), 20)

def test_callcc():
    assert_equal(callcc(lambda x: x)(lambda x: x)(10)(20), 20)



from nose.tools import *
from hawkweed.classes.list_cls import List

def test_repr():
    assert_equal(List([0]).__repr__(), "List([0])")

def test_str():
    x = List([0])
    assert_equal(x.__str__(), x.__repr__())



"""A collection base class"""
class Collection():
    """A collection base class that introduces nested getting and setting"""

    #The default value for get_in; normally None
    DFLT = None

    def get(self, index, dflt=None):
        """This method needs to be implemented to subclass Collection"""
        raise NotImplementedError("You need to implement get to use the Collection base class")

    def __setitem__(self, *args, **kwargs):
        raise NotImplementedError("You need to implement __setitem__ to use the Collection \
                                   base class")

    def get_in(self, *keys, **kwargs):
        """
        A getter for deeply nested values.

        Complexity: the complexity differs depending on the collection's lookup/get complexity
        params:
            *keys: they keys that should be worked through
            **kwargs: this only accepts dflt (the default element, normally DFLT), all others
                      will be ignored
        returns:
            the element that was found or the default element
        """
        key = keys[0]
        cont = len(keys) > 1

        elem = self.get(key)

        if elem is None:
            return kwargs.get("dflt", self.DFLT)

        if not cont or not hasattr(elem, "get_in"):
            return elem

        return elem.get_in(*keys[1:], **kwargs)

    def update_in(self, *keys, **kwargs):
        """
        A setter for deeply nested values.

        Complexity: the complexity differs depending on the collection's lookup/set complexity
        params:
            *keys: they keys that should be worked through
            **kwargs: this only accepts to (a needed argument that specifies what the element
                      should be set to), all others will be ignored
        returns: self
        throws: Whatever a collection returns when a key does not exist (mostly Index- or KeyError)
        """
        key = keys[0]
        cont = len(keys) > 1

        elem = self.get(key)

        if cont:
            elem.update_in(*keys[1:], **kwargs)
        else:
            self[key] = kwargs["to"]
        return self



from nose.tools import *
from hawkweed.functional.primitives import identity
from hawkweed.monads.listm import ListM

def test_unit():
    x = ListM.unit(1)
    assert_equal(next(x), 1)
    with assert_raises(StopIteration):
        next(x)

def test_bind():
    x = ListM(range(10))
    for i, x in enumerate(x.bind(identity)):
        assert_equal(i, x)



from nose.tools import *
from hawkweed.monads.monad import Monad, doM, wrapM
from hawkweed.monads.maybe import Just

def test_doM():
    def x():
        res1 = yield Just(1)
        res2 = yield Just(2)
        yield Just(res1 + res2)
    assert_equal(doM(x()), 3)

def test_wrapM():
    assert_equal(wrapM(Just(1)).imag, 0)
    assert_equal(wrapM(Just(1)).real, 1)
    assert_equal(wrapM(Just(1)).bind(lambda x: x + 1), 2)

def test_ret():
    assert_equal(Monad(1).ret(), 1)

def test_bind_sugar():
    with assert_raises(NotImplementedError):
        Monad(1) >> 1

def test_lift():
    liftM = Monad.lift(lambda x: x)
    assert_equal(liftM(1), Monad(1))



"""A Future class"""
from hawkweed.functional.primitives import reduce
from hawkweed.classes.repr import Repr

class Future(Repr):
    """A Future class"""
    def __init__(self, value):
        """
        Takes a binary function (taking success and error, respectively)
        and builds a Future from it.

        Complexity: O(1)
        params:
            value: the function to encase
        returns:
            a Future
        """
        self.value = value
        self.transforms = []

    @staticmethod
    def of(value):
        """
        Creates a Future from a static value, immediately returning it.

        Complexity: O(1)
        params:
            value: the value to encase
        returns:
            a Future
        """
        return Future(lambda res, rej: res(value))

    @staticmethod
    def reject(value):
        """
        Creates a Future from a static value, immediately rejecting it.

        Complexity: O(1)
        params:
            value: the value to encase
        returns:
            a Future
        """
        return Future(lambda res, rej: rej(value))

    @staticmethod
    def encase(fun, args=None):
        """
        Encases an ordinary function in a Future. If the function runs
        as expected the return value will be returned to the success
        callback. If an exception occurs it will be returned to the
        error callback.

        Special behaviour:
        You need to specify args. If the function does not have any,
        add args=[]. If you do not a function that takes arguments
        will be returned.

        Complexity: O(1)
        params:
            fun: the function to encase
            args: the arguments to pass to the function (defaults to None,
                  override to an empty sequence if no arguments are needed)
        returns:
            a Future
        """
        if args is None:
            return lambda *args: Future.encase(fun, args=args)
        def res(res, rej):
            """Internal encase function"""
            try:
                return res(fun(*args))
            except Exception as e:
                return rej(e)
        return Future(res)

    def __repr__(self):
        return "Future({})".format(self.value)

    def apply(self, fun):
        self.transforms.append(fun)
        return self

    def chain(self, future):
        """
        Chains a future to this one. This will intercept
        any calls to fork insofar as both Futures are chained
        before any call to the callbacks. Any error in both
        Futures will result in a call to the error callback.

        Complexity: O(1)
        params:
            future: the Future to chain
        returns:
            a Future
        """
        def chained(res, rej):
            """Internal chain function"""
            self.value(lambda x: future(x).fork(res, rej), rej)
        return Future(chained)

    def fork(self, res, err):
        """
        Registers resolvers for this Future.

        Complexity: O(1)
        params:
            res: the resolver function
            err: the error function
        returns:
            whatever the functions return
        """
        def resolver(trans):
            """Internal fork function that applies transformations"""
            try:
                return res(reduce(lambda acc, x: x(acc), trans, self.transforms))
            except Exception as e:
                if err:
                    return err(e)
                raise
        return self.value(resolver, err)



from nose.tools import *
from hawkweed.functional.logical import *

def test_all():
    assert_true(all(lambda x: x > 0, range(1,100)))
    assert_false(all(lambda x: x > 0, range(100,-1,-1)))

def test_any():
    assert_true(any(lambda x: x > 0, [0, 0, 0, 1, 0]))
    assert_false(any(lambda x: x > 0, [0, 0, 0, 0, -1]))

def test_complement():
    assert_false(complement(all, lambda x: x > 0, range(1, 3)))
    assert_true(complement(all, lambda x: x > 0, [0]))

def test_false():
    assert_false(false())

def test_true():
    assert_true(true())

def test_and():
    assert_true(and_(True)(True))
    assert_false(and_(False)(True))

def test_or():
    assert_true(or_(True)(True))
    assert_false(or_(False)(False))



from nose.tools import *
from hawkweed.classes.future import Future
from hawkweed.functional.primitives import curry
from hawkweed.functional.mathematical import inc

def test_of_fork():
    Future.of(1).fork(curry(assert_equal)(1), None)

def test_reject():
    Future.reject(1).fork(None, curry(assert_equal)(1))

def test_encase():
    Future.encase(inc, args=(0,)).fork(curry(assert_equal)(1), None)

def test_apply():
    Future.of(1).apply(inc).fork(curry(assert_equal)(2), None)

def test_chain():
    Future.of(1).chain(Future.encase(inc)).fork(curry(assert_equal)(2), None)



from itertools import chain
from nose.tools import *
from hawkweed.monads.either import Either, Left, Right, is_right,\
        is_left, is_either, either, lefts, rights, partition_eithers
from hawkweed.functional.primitives import identity

def test_right():
    assert_equal(Right(10).bind(identity), 10)

def test_nothing():
    l = Left("failure")
    assert_equal(l.bind(lambda _: "lol"), l)

def test_is_right():
    assert_true(is_right(Right(10)))
    assert_false(is_right(Left("no")))
    assert_false(is_right(10))

def test_is_left():
    assert_true(is_left(Left("yes")))
    assert_false(is_left(Right(10)))
    assert_false(is_left(10))

def test_is_either():
    assert_true(is_either(Right(10)))
    assert_true(is_either(Left("yes")))
    assert_false(is_either(10))

def test_either():
    v = "val"
    either(lambda x: assert_equal(Left(v), x), None, Left(v))
    either(None, lambda x: assert_equal(v, x), Right(v))
    with assert_raises(ValueError):
        either(None, None, 10)

def test_lefts():
    l = [Left("failure"), Left("i died"), Left("noes")]
    lr = l + [Right(1)]
    assert_equal(list(lefts(lr)), l)

def test_rights():
    r = [Right(x) for x in range(4)]
    rl = [Left("moo")] + r
    assert_equal(list(rights(rl)), r)

def test_partition_eithers():
    r = [Right(x) for x in range(4)]
    l = [Left(x) for x in ["failure"] * 4]
    rl = list(chain.from_iterable(zip(r, l)))
    assert_equal([list(x) for x in partition_eithers(rl)], [l, r])



from hawkweed.classes import List, Dict
from nose.tools import *

def test_get_in():
    assert_equal(List([Dict({1:1})]).get_in(0, 1), 1)

def test_get_in_not_in():
    assert_equal(List([Dict({1:1})]).get_in(1, 1), None)

def test_get_in_dflt():
    assert_equal(List([1]).get_in(1, "dflt"), "dflt")

def test_update_in():
    assert_equal(List([Dict({1:1})]).update_in(0, 1, to=2),
                 List([Dict({1:2})]))

def test_get_in_not_in():
    assert_equal(List([Dict({1:1})]).get_in(1, 1), None)

def test_get_in_dflt():
    assert_equal(List([1]).get_in(1, dflt="dflt"), "dflt")

def test_get_in_reset_dflt():
    x = List([1])
    x.DFLT = "dflt"
    assert_equal(x.get_in(1), "dflt")



"""A small collection of functions for logic."""
from hawkweed.functional.primitives import curry

@curry
def all(fun, values):
    """
    A function that will return True if the predicate is True for all
    provided values.

    Complexity: O(n)
    params:
        fun: the predicate function
        values: the values to test against
    returns: boolean
    """
    for element in values:
        if not fun(element):
            return False
    return True

@curry
def any(fun, values):
    """
    A function that will return True if the predicate is True for any
    of the provided values.

    Complexity: O(n)
    params:
        fun: the predicate function
        values: the values to test against
    returns: boolean
    """
    for element in values:
        if fun(element):
            return True
    return False

@curry
def and_(fst, snd):
    """Wraps the logical and function."""
    return fst and snd

@curry
def or_(fst, snd):
    """Wraps the logical and function."""
    return fst or snd

def not_(val):
    """Wraps the logical not function."""
    return not val

@curry
def complement(fun, *args, **kwargs):
    """
    Takes a function and it's args and returns it's complement.

    Complexity: O(1)
    params:
        fun: the function to complement
        *args: the arguments that should be passed to the function
        **kwaargs: the keyword arguments that should be passed to the function
    """
    return not fun(*args, **kwargs)

def false():
    """
    Always returns False.

    Complexity: O(1)
    returns: False
    """
    return False

def true():
    """
    Always returns True.

    Complexity: O(1)
    returns: True
    """
    return True



from hawkweed.classes import Set
from nose.tools import *

def test_clear():
    assert_equal(Set({1,2}).clear(), Set({}))

def test_remove():
    assert_equal(Set({1}).remove(1), Set({}))

def test_add():
    assert_equal(Set({}).add(1), Set({1}))

def test_update():
    assert_equal(Set({}).update({1}), Set({1}))

def test_reset():
    reset = Set({1})
    assert_equal(Set({1, 2, 3, 4}).reset(reset), reset)

def test_remove_empty():
    assert_equal(Set({1, 2, 3, None}).remove_empty(), Set({1, 2, 3}))

def test_remove_empty_custom_fun():
    assert_equal(Set({1, 2, 3, 4}).remove_empty(fun=lambda x: x != 4), Set({1, 2, 3}))



"""A collection of functional primitives"""
from functools import wraps, partial
from functools import reduce as _reduce
from inspect import getargspec


def curry(fun):
    """
    A working but dirty version of a currying function/decorator.
    """
    def _internal_curry(fun, original=None, given=0):
        if original is None:
            original = fun
        spec = getargspec(original)
        opt = len(spec.defaults or [])
        needed = len(spec.args) - given - opt
        @wraps(fun)
        def internal(*args, **kwargs):
            """The internal currying function"""
            if len(args) >= needed:
                return fun(*args, **kwargs)
            else:
                return _internal_curry(wraps(fun)(partial(fun, *args, **kwargs)),
                                       original,
                                       given=len(args))
        return internal
    return _internal_curry(fun)

@curry
def map(fun, values):
    """
    A function that maps a function to a list of values and returns the new generator.

    Complexity: O(n*k) where k is the complexity of the given function
    params:
        fun: the function that should be applied
        values: the list of values we should map over
    returns:
        the new generator
    """
    return (fun(value) for value in values)

@curry
def reduce(fun, init, values=None):
    """
    A function that reduces a list to a single value using a given function.

    Complexity: O(n*k) where k is the complexity of the given function
    params:
        fun: the function that should be applied
        values: the list of values we should reduce
    returns:
        the reduced value
    """
    if values is None:
        return _reduce(fun, init)
    else:
        return _reduce(fun, values, init)

@curry
def apply(fun, args, kwargs=None):
    """
    applies a list of arguments (and an optional dict of keyword arguments)
    to a function.

    Complexity: O(k) where k is the complexity of the given function
    params:
        fun: the function that should be applied
        args: the list of values we should reduce
    returns:
        the reduced value
    """
    if kwargs is None:
        kwargs = {}
    return fun(*args, **kwargs)

def pipe(*funs):
    """
    composes a bunch of functions. They will be applied one after the other.

    Complexity: depends on the given functions
    params:
        *funs: the functions that should be chained
    returns: the chained function
    """
    def internal(*args, **kwargs):
        """The internal piping function"""
        return reduce(lambda acc, fun: fun(acc),
                      funs[0](*args, **kwargs),
                      funs[1:])
    return internal

def compose(*funs):
    """
    composes a bunch of functions. They will be applied in reverse order
    (so this function is the reverse of pipe).

    Complexity: depends on the given functions
    params:
        *funs: the functions that should be chained
    returns: the chained function
    """
    return apply(pipe, reversed(funs))

def identity(value):
    """
    The identity function. Takes a value and returns it.

    Complexity: O(1)
    params:
        value: the value
    returns: the value
    """
    return value

@curry
def tap(fun, value):
    """
    A function that takes a function and a value, applies the function
    to the value and returns the value.

    Complexity: O(k) where k is the complexity of the given function
    params:
        fun: the function
        value: the value
    returns: the value
    """
    fun(value)
    return value

def constantly(value):
    """
    A generator that returns the given value forever.

    Complexity: O(1)
    params:
        value: the value to return
    returns: an infinite generator of the value
    """
    while True:
        yield value

def delay(fun, *args, **kwargs):
    """
    A function that takes a function and its arguments and delays its execution
    until it is needed. It also caches the executed return value and prevents
    it from being executed again (always returning the first result).

    params:
        fun: the function
        args: the function's args
        kwargs: the function's keyword arguments
    returns: the function result
    """
    # this is a horrible hack around Python 2.x's lack of nonlocal
    _int = ["__delay__unset"]
    @wraps(fun)
    def internal():
        """The internal delay function"""
        if _int[0] == "__delay__unset":
            _int[0] = fun(*args, **kwargs)
        return _int[0]
    return internal

@curry
def flip(fun, first, second, *args):
    """
    Takes a function and applies its arguments in reverse order.

    params:
        fun: the function
        first: the first argument
        second: the second argument
        args: the remaining args (this weird first, second, args thing
              is there to prevent preemptive passing of arguments)
    returns: the result of the function fun
    """
    return apply(fun, reversed(args + (first, second)))



from hawkweed.classes import Dict
from nose.tools import *

def test_reverse():
    assert_equal(Dict({1: 2, 3: 4}).reverse(), Dict({2: 1, 4: 3}))

def test_reset():
    reset = Dict({2: 1, 4: 3})
    assert_equal(Dict({1: 2, 3: 4}).reset(reset), reset)

def test_clear():
    assert_equal(Dict({1: 2, 3: 4}).clear(), Dict({}))

def test_setdefault():
    d = Dict({2: 1, 4: 3})
    assert_equal(d.setdefault(1), d)

def test_update():
    assert_equal(Dict({1: 2, 3: 4}).update({2: 3}), Dict({1: 2, 2: 3, 3: 4}))

def test_remove_empty():
    assert_equal(Dict({1: 2, 3: None}).remove_empty(), Dict({1: 2}))

def test_remove_empty_keys():
    assert_equal(Dict({1: 2, None: 3}).remove_empty(filter_keys=True), Dict({1: 2}))

def test_remove_empty_custom_fun():
    assert_equal(Dict({1: 2, 4: 3}).remove_empty(fun=lambda x: x != 3), Dict({1: 2}))

def test_reduce():
    assert_equal(Dict({1: 2, 3: 4}).reduce(lambda acc, k, v: acc + k + v, 0), 10)

def test_reduce_no_acc():
    assert_equal(Dict({1: 2, 3: 4}).reduce(lambda acc, k, v: acc + (k, v)), (1, 2, 3, 4))

def test_pick():
    d = Dict({1: 2, 3: 4, 5: 6})
    assert_equal(d.pick(1, 5), Dict({1: 2, 5: 6}))



from nose.tools import *
from hawkweed.monads.io import IO, CachedIO

def test_io():
    with open("tests/test_file.txt") as f:
        linegen = IO(f.readline)
        res1 = linegen.ret()
        res2 = linegen.ret()
        assert_not_equal(res1, res2)

def test_cached_io():
    with open("tests/test_file.txt") as f:
        linegen = CachedIO(f.readline)
        res1 = linegen.ret()
        res2 = linegen.ret()
        assert_equal(res1, res2)



"A collection of augmented standard classes"
from hawkweed.classes.list_cls import List
from hawkweed.classes.dict_cls import Dict
from hawkweed.classes.iterable import Iterable
from hawkweed.classes.set_cls import Set
from hawkweed.classes.repr import Repr
from hawkweed.classes.collection import Collection



from functools import wraps
from nose.tools import assert_equal, assert_raises
from hawkweed.classes import List

def prepare(f):
    def internal():
        x = List([1,2,3,4])
        f(x)
    return internal

@prepare
def test_concatenate(x):
    y = List([5,6,7,8])
    for i, x in enumerate(x.concatenate(y)):
        assert_equal(i+1, x)

@prepare
def test_concatenate_multiple(x):
    y = List([5,6,7,8])
    z = List([9,10])
    for i, x in enumerate(x.concatenate(y, z)):
        assert_equal(i+1, x)

@prepare
def test_concatenate_none(x):
    for i, x in enumerate(x.concatenate()):
        assert_equal(i+1, x)

def test_nth():
    x = List(range(100))
    for i in x.nth(2):
        assert_equal(i % 2, 0)

def test_nth_1():
    x = List(range(100))
    for i, x in enumerate(x.nth(1)):
        assert_equal(i, x)

@prepare
def test_nth_smaller_than_0(x):
    with assert_raises(ValueError) as ve:
        x.nth(0)
    assert_equal(str(ve.exception), "step size must be bigger than 0, was 0")
    with assert_raises(ValueError) as ve:
        x.nth(-1)
    assert_equal(str(ve.exception), "step size must be bigger than 0, was -1")



from hawkweed.classes import List
from nose.tools import *

def test_clear():
    assert_equal(List([1,2]).clear(), List([]))

def test_append():
    assert_equal(List([1,2]).append(3), List([1, 2, 3]))

def test_extend():
    assert_equal(List([1,2]).extend([3]), List([1, 2, 3]))

def test_get():
    assert_equal(List([1]).get(0), 1)

def test_get_not_in():
    assert_equal(List([1]).get(1), None)

def test_get_dflt():
    assert_equal(List([1]).get(1, "dflt"), "dflt")

def test_reset():
    reset = List([1])
    assert_equal(List([1, 2, 3, 4]).reset(reset), reset)

def test_remove_empty():
    assert_equal(List([1, 2, 3, None]).remove_empty(), List([1, 2, 3]))

def test_remove_empty_custom_fun():
    assert_equal(List([1, 2, 3, 4]).remove_empty(fun=lambda x: x != 4), List([1, 2, 3]))

def test_take():
    assert_equal(List(List(range(10)).take(5)), List(range(5)))

def test_take_iterate():
    for i, e in enumerate(List(range(10)).take(5)):
        assert_equal(i, e)

def test_drop():
    assert_equal(List(List(range(10)).drop(5)), List(range(5,10)))

def test_drop_iterate():
    dropped = 5
    for i, e in enumerate(List(range(10)).drop(dropped)):
        assert_equal(i + dropped, e)

def test_take_while():
    assert_equal(List(List(range(10)).take_while(lambda x: x < 5)), List(range(5)))

def test_take_while_iterate():
    for i, e in enumerate(List(range(10)).take_while(lambda x: x < 5)):
        assert_equal(i, e)

def test_drop_while():
    assert_equal(List(List(range(10)).drop_while(lambda x: x < 5)), List(range(5,10)))

def test_drop_while_iterate():
    dropped = 5
    for i, e in enumerate(List(range(10)).drop_while(lambda x: x < dropped)):
        assert_equal(i + dropped, e)

def test_rindex():
    assert_equal(List(range(10)).rindex(8), 1)



"""An augmented version of the set type."""
from hawkweed.classes.repr import Repr

class Set(Repr, set):
    """An augmented version of the set type."""
    def reset(self, other):
        """
        Resets a set to another sets content

        Complexity: O(n)
        params:
            other: the set this set should be resetted to
        returns: self
        """
        self.clear()
        self.update(other)
        return self

    def remove_empty(self, fun=None):
        """
        Removes empty elements from the list.

        Complexity: O(n+k) where k is the number of things to remove
        params:
            fun: a function that takes an element and returns whether
                 it should be kept (defaults to bool())
        returns: self
        """
        if not fun:
            fun = bool

        rem = []
        for elem in self:
            if not fun(elem):
                rem.append(elem)

        for elem in rem:
            self.remove(elem)

        return self

    def clear(self):
        """
        Augmented clear function that returns self.

        returns: self
        """
        super(Set, self).clear()
        return self

    def remove(self, *args):
        """
        Augmented remove function that returns self.

        returns: self
        """
        super(Set, self).remove(*args)
        return self

    def add(self, *args):
        """
        Augmented add function that returns self.

        returns: self
        """
        super(Set, self).add(*args)
        return self

    def update(self, *args):
        """
        Augmented update function that returns self.

        returns: self
        """
        super(Set, self).update(*args)
        return self



"""The continuation monad"""
from hawkweed.monads.monad import Monad

class Continuation(Monad):
    """The continuation monad"""
    def __call__(self, cont):
        """
        Calls the monad with a continuation.

        Complexity: O(k) where k is the complexity of the monad value
        params:
            cont: the continuation
        returns: the result of the application
        """
        return self.value(cont)

    def bind(self, fun):
        """
        The monadic bind function of Continuation.
        Returns a function that takes a continuation.

        Complexity: O(k) where k is the complexity of the monad value
        params:
            fun: the function
        returns:
            the deferred function
        """
        return lambda cont: self.value(lambda x: fun(x)(cont))

def callcc(cc):
    """
    call with current continuation.

    Complexity: varies
    params:
        cc: the current continuation
    returns: the continuation
    """
    fun = lambda cont: cc(lambda val: Continuation(lambda _: cont(val)))(cont)
    return Continuation(fun)



"""A few mathematical functions"""
from hawkweed.functional.primitives import curry

def inc(num):
    """
    A function that increments its argument by 1.

    Complexity: O(1)
    params:
        num: the number to increment
    returns: the incremented number
    """
    return num + 1

def dec(num):
    """
    A function that decrements its argument by 1.

    Complexity: O(1)
    params:
        num: the number to decrement
    returns: the decremented number
    """
    return num - 1

@curry
def add(fst, snd):
    """This is a functional wrapper around the + operator"""
    return fst + snd

@curry
def sub(fst, snd):
    """This is a functional wrapper around the - operator"""
    return fst - snd

@curry
def mul(fst, snd):
    """This is a functional wrapper around the * operator"""
    return fst * snd

@curry
def div(fst, snd):
    """This is a functional wrapper around the / operator"""
    return fst / snd

@curry
def mod(fst, snd):
    """This is a functional wrapper around the % operator"""
    return fst % snd

@curry
def clamp(frm, to, value):
    """
    Clamps an ordinable type between two others.

    Complexity: O(1)
    params:
        frm: the lower end
        to: the upper end
        value: the value
    returns: the clamped value
    """
    if frm > to:
        raise ValueError("frm cannot be bigger than to in clamp")
    if value > to:
        return to
    if value < frm:
        return frm
    return value



"""The list monad"""
from hawkweed.monads.monad import Monad

class ListM(Monad):
    """The list monad"""

    @staticmethod
    def unit(val):
        """
        takes one value and returns a generator that yields that value.

        Complexity: O(1)
        params:
            val: the value to yield
        returns:
            the generator of the parameter
        """
        yield val

    def bind(self, fun):
        """
        The monadic bind function of ListM.
        It will apply the function to all the elements of the
        value and return a generator.

        Complexity: O(k*n) where k is the complexity of the function
        params:
            fun: the function
        returns:
            the generator of the transformed values
        """
        for element in self.value:
            yield fun(element)



"""A few monads"""
from hawkweed.monads.monad import Monad
from hawkweed.monads.listm import ListM
from hawkweed.monads.continuation import Continuation
from hawkweed.monads.identity import Identity
from hawkweed.monads.maybe import *
from hawkweed.monads.io import IO, CachedIO
from hawkweed.monads.either import either



# coding=utf-8
import os

from setuptools import find_packages
from setuptools import setup

with open('README.rst') as readme:
    long_description = readme.read()

setup(
    name = "hawkweed",
    version = "0.1.2",
    description = "Extending Python builtin types",
    long_description = long_description,
    author = "Veit Heller",
    author_email = "veit@veitheller.de",
    license = "MIT License",
    url = "https://github.com/hellerve/hawkweed",
    download_url = 'https://github.com/hellerve/hawkweed/tarball/0.1.2',
    packages = find_packages(),
    include_package_data = True,
)



from types import GeneratorType
from nose.tools import *
from hawkweed.functional.primitives import identity
from hawkweed.monads.maybe import *

def test_just():
    assert_equal(Just(10).bind(identity), 10)

def test_nothing():
    assert_equal(Nothing().value, None)

def test_chain():
    assert_equal(Just(10).bind(lambda val: Just(11).bind(lambda val2: val + val2)), 21)
    assert_equal(Just(10).bind(lambda val: Nothing().bind(lambda val2: val + val)), Nothing())

def test_is_just():
    assert_true(is_just(Just(1)))
    assert_false(is_just(Nothing()))

def test_is_nothing():
    assert_true(is_nothing(Nothing()))
    assert_false(is_nothing(Just(1)))

def test_is_maybe():
    assert_true(is_maybe(Just(1)))
    assert_true(is_maybe(Nothing()))
    assert_false(is_maybe(object()))

def test_from_just():
    assert_equal(from_just(Just(1)), 1)
    with assert_raises(ValueError):
        from_just(Nothing())
    with assert_raises(ValueError):
        from_just(1)

def test_from_maybe():
    assert_equal(from_maybe(Just(1), 100), 1)
    assert_equal(from_maybe(Just(1)), 1)
    assert_equal(from_maybe(Nothing(), 100), 100)
    assert_equal(from_maybe(Nothing()), None)
    with assert_raises(ValueError):
        from_maybe(1)

def test_list_to_maybe():
    assert_equal(list_to_maybe([]), Nothing())
    assert_equal(list_to_maybe([1]), Just(1))
    assert_equal(list_to_maybe([1, 10]), Just(1))
    with assert_raises(ValueError):
        list_to_maybe(1)

def test_maybe_to_list():
    assert_equal(maybe_to_list(Nothing()), [])
    assert_equal(maybe_to_list(Just(1)), [1])
    with assert_raises(ValueError):
        maybe_to_list(1)

def test_value_to_maybe():
    assert_equal(value_to_maybe([]), Nothing())
    assert_equal(value_to_maybe([1]), Just([1]))
    assert_equal(value_to_maybe(1), Just(1))
    assert_equal(value_to_maybe(0), Nothing())

def test_cat_maybe():
    assert_equal(list(cat_maybe([Just(1), Nothing(), Just(2)])),
                 [Just(1), Just(2)])

def test_map_maybe():
    assert_equal(list(map_maybe(identity, [Just(1), Nothing(), Just(2)])),
                      [Just(1), Just(2)])
    assert_is_instance(map_maybe(identity, [Just(1)]), GeneratorType)



"""The monad base class"""
from hawkweed.functional.primitives import apply

class Monad(object):
    """The monad base class"""
    def __init__(self, value):
        self.value = value

    def __repr__(self):
        return "{}({})".format(self.__class__.__name__, self.value)

    def __str__(self):
        return self.__repr__

    def ret(self):
        """
        Returns the monad value.

        Complexity: O(1)
        returns: the monads value
        """
        return self.value

    def bind(self, fun):
        """
        The bind function. Needs to be implemented by any monad.

        Complexity: varies
        params:
            fun: the function to bind ti the monad value
        returns:
            the manipulated monad value
        """
        raise NotImplementedError("Monads need to implement the bind function")

    def __rshift__(self, val):
        """
        The 'bind' operator. Syntactic sugar for binding.
        """
        return self.bind(val)

    @classmethod
    def lift(cls, fun):
        """
        Lifts a function into the monad.

        Complexity: O(1)
        params:
            fun: the function to lift
        returns: a function that funnels values into the given
                 function and returns the result wrapped in a monad
        """
        def lifter(*args, **kwargs):
            "The lifting function"""
            return cls(apply(fun, args, kwargs))
        return lifter

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.value == other.value

    def __ne__(self, other):
        return not self.__eq__(other)

def doM(fun):
    """
    Takes a generator and unpacks values for every yield.

    Complexity: O(k) where k is the complexity of fun
    params:
        fun: the generator
    returns: the return value of the generator
    """
    def internal(value):
        """The step function"""
        try:
            res = fun.send(value).value
            return internal(res)
        except StopIteration:
            return value
    return internal(next(fun).value)

def wrapM(monad):
    """
    Takes a monad and wraps it so that any attribute that is not
    in the monad is automaticall forwarded to the monad value.

    Complexity: O(1)
    params:
        monad: the monad to wrap
    returns: the proxy object
    """
    class Proxy():
        """The monad proxy object"""
        def __init__(self, monad):
            self.monad = monad

        def __getattr__(self, prop):
            if hasattr(self.monad, prop):
                return getattr(self.monad, prop)
            elif hasattr(self.monad.value, prop):
                return getattr(self.monad.value, prop)
            raise AttributeError(prop)

    return Proxy(monad)



"""A few utility functions concerning indexable things"""
from hawkweed.functional.primitives import curry
from hawkweed.functional.mathematical import inc, dec

def first(element):
    """
    A wrapper around element[0].

    params:
        element: an element that implements __getitem__
    """
    return element[0]

def rest(element):
    """
    A wrapper around element[1:].

    params:
        element: an element that implements __getitem__
    """
    return element[1:]

def second(element):
    """
    A wrapper around element[1].

    params:
        element: an element that implements __getitem__
    """
    return element[1]

def last(element):
    """
    A wrapper around element[-1].

    params:
        element: an element that implements __getitem__
    """
    return element[-1]

@curry
def get(index, element):
    """
    A wrapper around element[index].

    params:
        index: the index at which we should get
        element: an element that implements __getitem__
    """
    return element[index]

@curry
def remove_from(index, element):
    """
    A wrapper around del element[index].

    params:
        index: the index at which we should delete
        element: an element that implements __delitem__
    """
    del element[index]
    return element

@curry
def remove_from_keep(index, element):
    """
    Non-destructively deletes the element at index index.

    Complexity: O(n)
    params:
        index: the index at which we should delete
        element: an element that implements __delitem__
    returns: the new list
    """
    return element[:index] + element[inc(index):]

@curry
def aperture(n, l):
    """
    Creates a generator of consecutive sublists of size n.
    If n is bigger than the length of the list, the
    generator will be empty.

    Complexity: O(slice_size*n)
    params:
        n: the slice size
        l: the list we should create the generator from
    returns the generator
    """
    index = 0
    stop = len(l) - dec(n)
    while index < stop:
        yield l[index:index+n]
        index += 1



"""An augmented version of the dict class"""
from hawkweed.computed import PY3
from hawkweed.classes.iterable import Iterable
from hawkweed.classes.repr import Repr
from hawkweed.classes.collection import Collection

class Dict(Repr, dict, Iterable, Collection):
    """An augmented version of the dict class"""
    def reset(self, other):
        """
        Resets a dict to another dicts content

        Complexity: O(n)
        params:
            other: the dict this dict should be resetted to
        returns: self
        """
        self.clear()
        self.update(other)
        return self

    def reverse(self):
        """
        Exchanges keys and values; if there are duplicate values, the
        last value that's been written wins.

        Complexity: O(2*n)
        returns: self
        """
        tmp = dict()
        for key, val in self.items():
            tmp[val] = key
        self.reset(tmp)
        del tmp
        return self

    def remove_empty(self, fun=None, filter_keys=False):
        """
        Removes empty pairs from the dict.

        Complexity: O(2*n)
        params:
            fun: a function that takes an element and returns whether it
                 should be kept (defaults to bool())
            filter_keys: a flag that indicates that filtering should be
                         done by keys, not by values
        returns: self
        """
        if not fun:
            fun = bool
        tmp = dict()
        for key, val in self.items():
            checker = key if filter_keys else val
            if fun(checker):
                tmp[key] = val
        self.reset(tmp)
        del tmp
        return self

    def update(self, *args, **kwargs):
        """
        Update dictionary. Same as in dict(), but returns self.

        returns: self
        """
        super(Dict, self).update(*args, **kwargs)
        return self

    def reduce(self, fun, acc=None):
        """
        Reduce the dict to a value (using function fun).

        Complexity: O(n)
        params:
            fun: a function that takes the accumulator and current key
                 and value and returns the new accumulator
            acc: the initial accumulator (defaults to tuple of first
                 key and value taken from the iterator)
        returns: self
        """
        iterator = self.items().__iter__()
        if acc is None:
            acc = iterator.__next__() if PY3 else iterator.next()
        for key, val in iterator:
            acc = fun(acc, key, val)
        return acc

    def clear(self):
        """
        Augmented clear function that returns self.

        returns: self
        """
        super(Dict, self).clear()
        return self

    def setdefault(self, *args, **kwargs):
        """
        Augmented setdefault function that returns self.

        returns: self
        """
        super(Dict, self).setdefault(*args, **kwargs)
        return self

    def pick(self, *keys):
        """
        Takes a list of keys to pick and returns a subdict that contains
        only those entries.

        Complexity: O(k) where k is the number of keys
        params:
            *keys: the keys to pick
        returns: the subdict
        """
        newd = {}

        for key in keys:
            if key in self:
                newd[key] = self[key]

        return newd



import time
import math
from nose.tools import *
from hawkweed.functional.primitives import *

def test_curry():
    assert_equal(curry(lambda x, y: x + y)(1)(2), 3)
    assert_equal(curry(lambda x, y: x + y)(1, 2), 3)

def test_curry_decorator():
    @curry
    def add(x, y):
        return x + y
    assert_equal(add(1, 2), 3)
    assert_equal(add(1)(2), 3)
    assert_equal(add.__name__, "add")

def test_map():
    dbl = map(lambda x: x + x)
    assert_equal(list(dbl([1, 2, 3])), [2, 4, 6])

def test_reduce():
    sm = reduce(lambda acc, x: acc + x)
    assert_equal(sm([1,2,3,4]), 10)
    assert_equal(sm(10, [1,2,3,4]), 20)

def test_apply():
    sum3 = lambda x, y, z: x + y + z
    assert_equal(apply(sum3, [1, 2, 3]), sum3(1, 2, 3))
    sum3 = lambda x, y, z=0: x + y + z
    assert_equal(apply(sum3, [1, 2], {"z": 2}), sum3(1, 2, z=2))

def test_pipe():
    add1 = lambda x: x + 1
    assert_equal(pipe(add1, add1)(2), 4)
    fun1 = lambda x: x[1]
    fun2 = lambda x: x[0]
    assert_equal(pipe(fun1, fun2)([1,[4]]), 4)

def test_compose():
    add1 = lambda x: x + 1
    assert_equal(pipe(add1, add1)(2), compose(add1, add1)(2))
    fun1 = lambda x: x[1]
    fun2 = lambda x: x[0]
    assert_equal(compose(fun2, fun1)([1,[4]]), 4)

def test_identity():
    assert_equal(identity([]), [])
    assert_equal(identity(1), 1)

def test_tap():
    assert_equal(tap(identity, 1), 1)
    assert_equal(tap(identity)(1), 1)

def test_constantly():
    for i, n in enumerate(constantly(1)):
        if i > 99:
            break
        assert_equal(n, 1)

def test_delay():
    # horrible test for a horrible function, heh
    x = delay(range, 1, 10000)
    before1 = time.clock()
    x()
    after1 = time.clock()
    before2 = time.clock()
    x()
    after2 = time.clock()
    assert_true((after1 - before1) > (after2 - before2))

def test_flip():
    test = lambda x, y: [x, y]
    args = [1, 2]
    assert_equal(apply(flip(test), args), list(reversed(args)))
